# Tests for contract related reports
import os
from datetime import datetime, timedelta
from zc.async.testing import wait_for_result
from zope.interface.verify import verifyClass, verifyObject
from zope.component import getUtility
from waeup.ikoba.interfaces import IJobManager, IIkobaUtils
from waeup.ikoba.customers.reports.contract_statistics import (
    get_contract_stats, ContractStatisticsReport, IContractStatisticsReport)
from waeup.ikoba.customers.tests.test_catalog import CatalogTestSetup
from waeup.ikoba.customers.tests.test_browser import CustomersFullSetup
from waeup.ikoba.testing import FunctionalLayer
from waeup.ikoba.tests.test_async import FunctionalAsyncTestCase
from waeup.ikoba.browser.tests.test_pdf import samples_dir

class ContractStatisticsReportTests(CatalogTestSetup):

    layer = FunctionalLayer

    states = ('created', 'awaiting payment',
              'submitted for approval',
              'approved', 'expired',
              'rejected', 'Total')

    def test_iface(self):
        # ensure we fullfill interface contracts
        obj = ContractStatisticsReport()
        verifyClass(IContractStatisticsReport, ContractStatisticsReport)
        verifyObject(IContractStatisticsReport, obj)
        return

    def test_get_contract_stats(self):
        # we can get a table with one contract
        result1 = get_contract_stats()
        self.assertEqual(
            result1,
            ((u'Sample Contract', u'All'),
             self.states,
             ((1, 0, 0, 0, 0, 0, 1),
              (1, 0, 0, 0, 0, 0, 1),))
            )
        return

    def test_create_pdf(self):
        report = ContractStatisticsReport()
        result = report.create_pdf()
        self.assertTrue(result.startswith('%PDF-'))
        path = os.path.join(samples_dir(), 'contract_statistics.pdf')
        open(path, 'wb').write(result)
        print "Sample PDF contract_statistics.pdf written to %s" % path
        return

class ContractStatisticsReportUITests(CustomersFullSetup, FunctionalAsyncTestCase):

    layer = FunctionalLayer

    def wait_for_report_job_completed(self):
        # helper function waiting until the current export job is completed
        manager = getUtility(IJobManager)
        job_id = self.app['reports'].running_report_jobs[0][0]
        job = manager.get(job_id)
        wait_for_result(job)
        return job_id

    def stored_in_reports(self, job_id):
        # tell whether job_id is stored in reports's running jobs list
        for entry in list(self.app['reports'].running_report_jobs):
            if entry[0] == job_id:
                return True
        return False

    def trigger_report_creation(self):
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open('http://localhost/app/reports')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.browser.getLink("Create new report").click()
        self.browser.getControl(name="generator").value = ['contract_stats']
        self.browser.getControl("Configure").click()
        #self.browser.getControl(name="session").value = ['2004']
        self.browser.getControl("Create").click()
        return

    def test_report_download(self):
        # We can download a generated report
        self.trigger_report_creation()
        # When the job is finished and we reload the page...
        job_id = self.wait_for_report_job_completed()
        self.browser.open('http://localhost/app/reports')
        # ... the pdf file can be downloaded ...
        self.browser.getControl("Download").click()
        self.assertEqual(self.browser.headers['content-type'],
                         'application/pdf')
        self.assertTrue(
            'filename="ContractStatisticsReport' in
            self.browser.headers['content-disposition'])
        self.assertEqual(len(self.app['reports'].running_report_jobs), 1)
        job_id = self.app['reports'].running_report_jobs[0][0]
        # ... and discarded
        self.browser.open('http://localhost/app/reports')
        self.browser.getControl("Discard").click()
        self.assertEqual(len(self.app['reports'].running_report_jobs), 0)
        # Creation, downloading and discarding is logged
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'main.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'INFO - zope.mgr - customers.reports.contract_statistics.ContractStatisticsReportGeneratorPage - '
            'report %s created: Contract Statistics'
            % job_id in logcontent
            )
        self.assertTrue(
            'INFO - zope.mgr - customers.reports.contract_statistics.ContractStatisticsReportPDFView - '
            'report %s downloaded: ContractStatisticsReport'
            % job_id in logcontent
            )
        self.assertTrue(
            'INFO - zope.mgr - browser.reports.ReportsContainerPage - '
            'report %s discarded' % job_id in logcontent
            )
        return

    def test_report_purge(self):
        self.trigger_report_creation()
        job_id = self.wait_for_report_job_completed()
        self.browser.open('http://localhost/app/reports')
        self.browser.getControl("Purge").click()
        self.assertTrue('0 report(s) purged' in self.browser.contents)
        job_id, gen_name, user = self.app['reports'].get_running_report_jobs(user_id=None)[0]
        job = getUtility(IJobManager).get(job_id)
        tz = getUtility(IIkobaUtils).tzinfo
        delta = timedelta(weeks=5)
        setattr(job, '_begin_after', datetime.now(tz) - delta)
        self.browser.getControl("Purge").click()
        self.assertTrue('1 report(s) purged' in self.browser.contents)
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'main.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'INFO - zope.mgr - browser.reports.ReportsContainerPage - '
            'report %s purged' % job_id in logcontent
            )
        return